@@ -64,6 +64,7 @@ _append_args_with_separator(gint argc, GString *argv[], GString *result, gchar s
 #include "tf-template.c"
 #include "context-funcs.c"
 #include "fname-funcs.c"
+#include "tf-iterate.c"
 
 static Plugin basicfuncs_plugins[] =
 {
@@ -103,6 +104,7 @@ static Plugin basicfuncs_plugins[] =
   TEMPLATE_FUNCTION_PLUGIN(tf_list_slice, "list-slice"),
   TEMPLATE_FUNCTION_PLUGIN(tf_list_count, "list-count"),
   TEMPLATE_FUNCTION_PLUGIN(tf_list_append, "list-append"),
+  TEMPLATE_FUNCTION_PLUGIN(tf_list_search, "list-search"),
 
   /* numeric-funcs */
   TEMPLATE_FUNCTION_PLUGIN(tf_num_plus, "+"),
@@ -121,13 +123,18 @@ static Plugin basicfuncs_plugins[] =
   /* ip-funcs */
   TEMPLATE_FUNCTION_PLUGIN(tf_ipv4_to_int, "ipv4-to-int"),
   TEMPLATE_FUNCTION_PLUGIN(tf_indent_multi_line, "indent-multi-line"),
+  TEMPLATE_FUNCTION_PLUGIN(tf_dns_resolve_ip, "dns-resolve-ip"),
 
   /* misc funcs */
   TEMPLATE_FUNCTION_PLUGIN(tf_env, "env"),
   TEMPLATE_FUNCTION_PLUGIN(tf_template, "template"),
   TEMPLATE_FUNCTION_PLUGIN(tf_urlencode, "url-encode"),
   TEMPLATE_FUNCTION_PLUGIN(tf_urldecode, "url-decode"),
-  TEMPLATE_FUNCTION_PLUGIN(tf_base64encode, "base64-encode")
+  TEMPLATE_FUNCTION_PLUGIN(tf_base64encode, "base64-encode"),
+
+  /* functional */
+  TEMPLATE_FUNCTION_PLUGIN(tf_iterate, "iterate"),
+
 };
 
 gboolean
