@@ -145,14 +145,6 @@ http_dd_set_ca_dir(LogDriver *d, const gchar *ca_dir)
   self->ca_dir = g_strdup(ca_dir);
 }
 
-void
-http_dd_set_use_system_cert_store(LogDriver *d, gboolean enable)
-{
-  HTTPDestinationDriver *self = (HTTPDestinationDriver *) d;
-
-  self->use_system_cert_store = enable;
-}
-
 void
 http_dd_set_ca_file(LogDriver *d, const gchar *ca_file)
 {
@@ -189,7 +181,7 @@ http_dd_set_cipher_suite(LogDriver *d, const gchar *ciphers)
   self->ciphers = g_strdup(ciphers);
 }
 
-void
+gboolean
 http_dd_set_ssl_version(LogDriver *d, const gchar *value)
 {
   HTTPDestinationDriver *self = (HTTPDestinationDriver *) d;
@@ -220,32 +212,38 @@ http_dd_set_ssl_version(LogDriver *d, const gchar *value)
       /* SSL 3 only */
       self->ssl_version = CURL_SSLVERSION_SSLv3;
     }
-#ifdef CURL_SSLVERSION_TLSv1_0
+#if SYSLOG_NG_HAVE_DECL_CURL_SSLVERSION_TLSV1_0
   else if (strcmp(value, "tlsv1_0") == 0)
     {
       /* TLS 1.0 only */
       self->ssl_version = CURL_SSLVERSION_TLSv1_0;
     }
 #endif
-#ifdef CURL_SSLVERSION_TLSv1_1
+#if SYSLOG_NG_HAVE_DECL_CURL_SSLVERSION_TLSV1_1
   else if (strcmp(value, "tlsv1_1") == 0)
     {
       /* TLS 1.1 only */
       self->ssl_version = CURL_SSLVERSION_TLSv1_1;
     }
 #endif
-#ifdef CURL_SSLVERSION_TLSv1_2
+#if SYSLOG_NG_HAVE_DECL_CURL_SSLVERSION_TLSV1_2
   else if (strcmp(value, "tlsv1_2") == 0)
     {
       /* TLS 1.2 only */
       self->ssl_version = CURL_SSLVERSION_TLSv1_2;
     }
 #endif
-  else
+#if SYSLOG_NG_HAVE_DECL_CURL_SSLVERSION_TLSV1_3
+  else if (strcmp(value, "tlsv1_3") == 0)
     {
-      msg_warning("curl: unsupported SSL version",
-                  evt_tag_str("ssl_version", value));
+      /* TLS 1.3 only */
+      self->ssl_version = CURL_SSLVERSION_TLSv1_3;
     }
+#endif
+  else
+    return FALSE;
+
+  return TRUE;
 }
 
 void
