@@ -48,7 +48,6 @@ typedef struct
   HttpAuthHeader *auth_header;
   gchar *user_agent;
   gchar *ca_dir;
-  gboolean use_system_cert_store;
   gchar *ca_file;
   gchar *cert_file;
   gchar *key_file;
@@ -83,12 +82,11 @@ void http_dd_set_auth_header(LogDriver *d, HttpAuthHeader *auth_header);
 void http_dd_set_body(LogDriver *d, LogTemplate *body);
 void http_dd_set_accept_redirects(LogDriver *d, gboolean accept_redirects);
 void http_dd_set_ca_dir(LogDriver *d, const gchar *ca_dir);
-void http_dd_set_use_system_cert_store(LogDriver *d, gboolean enable);
 void http_dd_set_ca_file(LogDriver *d, const gchar *ca_file);
 void http_dd_set_cert_file(LogDriver *d, const gchar *cert_file);
 void http_dd_set_key_file(LogDriver *d, const gchar *key_file);
 void http_dd_set_cipher_suite(LogDriver *d, const gchar *ciphers);
-void http_dd_set_ssl_version(LogDriver *d, const gchar *value);
+gboolean http_dd_set_ssl_version(LogDriver *d, const gchar *value);
 void http_dd_set_peer_verify(LogDriver *d, gboolean verify);
 void http_dd_set_timeout(LogDriver *d, glong timeout);
 void http_dd_set_batch_bytes(LogDriver *d, glong batch_bytes);
